package main

import "testing"

func TestEndpointTreeFromSpanTreeMultiAppAlias(t *testing.T) {
	aa := "aAlias"
	ba := "bAlias"
	rootSpan := Span2Cal{
		ServiceName: "A",
		AppAlias:    ba,
		children: []*Span2Cal{
			{
				ServiceName: "B",
				AppAlias:    aa,
				children: []*Span2Cal{
					{
						ServiceName: "C",
						AppAlias:    aa,
						children:    nil,
					},
				},
			},
			{
				ServiceName: "A",
				AppAlias:    aa,
				children: []*Span2Cal{
					{
						ServiceName: "E",
						AppAlias:    aa,
						children:    nil,
					},
					{
						ServiceName: "G",
						AppAlias:    aa,
						children:    nil,
					},
				},
			},
			{
				ServiceName: "A",
				AppAlias:    aa,
				children: []*Span2Cal{
					{
						ServiceName: "A",
						AppAlias:    aa,
						children: []*Span2Cal{
							{
								ServiceName: "F",
								AppAlias:    aa,
								children:    nil,
							},
						},
					},
				},
			},
			{
				ServiceName: "A",
				AppAlias:    aa,
				children: []*Span2Cal{
					{
						ServiceName: "H",
						AppAlias:    ba,
						children: []*Span2Cal{
							{
								ServiceName: "I",
								AppAlias:    ba,
								children:    nil,
							},
						},
					},
				},
			},
		},
	}
	eps := EndpointTreeFromSpanTree(&rootSpan, nil, aa)
	if len(eps) != 4 {
		t.Fatalf("len(eps) = %d, want %d, eps:%+v", len(eps), 4, eps)
	}
	t.Logf("endpoint tree:%+v", eps[0])
}
func TestEndpointTreeFromSpanTree(t *testing.T) {
	appAlias := "ABC"
	rootSpan := Span2Cal{
		ServiceName: "A",
		AppAlias:    appAlias,
		children: []*Span2Cal{
			{
				ServiceName: "B",
				AppAlias:    appAlias,
				children: []*Span2Cal{
					{
						ServiceName: "C",
						AppAlias:    appAlias,
						children:    nil,
					},
				},
			},
			{
				ServiceName: "A",
				AppAlias:    appAlias,
				children: []*Span2Cal{
					{
						ServiceName: "E",
						AppAlias:    appAlias,
						children:    nil,
					},
					{
						ServiceName: "G",
						AppAlias:    appAlias,
						children:    nil,
					},
				},
			},
			{
				ServiceName: "A",
				AppAlias:    appAlias,
				children: []*Span2Cal{
					{
						ServiceName: "A",
						AppAlias:    appAlias,
						children: []*Span2Cal{
							{
								ServiceName: "F",
								AppAlias:    appAlias,
								children:    nil,
							},
						},
					},
				},
			},
			{
				ServiceName: "A",
				children: []*Span2Cal{
					{
						ServiceName: "H",
						AppAlias:    appAlias,
						children: []*Span2Cal{
							{
								ServiceName: "I",
								AppAlias:    appAlias,
								children:    nil,
							},
						},
					},
				},
			},
		},
	}
	eps := EndpointTreeFromSpanTree(&rootSpan, nil, appAlias)
	if len(eps) != 1 {
		t.Fatalf("len(eps) = %d, want %d, eps:%+v", len(eps), 1, eps)
	}
	t.Logf("endpoint tree:%+v", eps[0])
}

func TestSubTreesWithEndpointRoot(t *testing.T) {
	root := ServerEndpointNode{
		ServerName: "A",
		Endpoint:   "",
		children: []*ServerEndpointNode{
			{
				ServerName: "B2",
				Endpoint:   "",
				children: []*ServerEndpointNode{
					{
						ServerName: "C2",
						Endpoint:   "c2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "D2",
								Endpoint:   "d2.endpoint",
								children:   nil,
							},
						},
					},
					{
						ServerName: "E2",
						Endpoint:   "e2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "f2",
								Endpoint:   "f2.endpoint",
								children:   nil,
							},
						},
					},
				},
			},
			{
				ServerName: "B1",
				Endpoint:   "b1.endpoint",
				children: []*ServerEndpointNode{
					{
						ServerName: "C1",
						Endpoint:   "",
						children: []*ServerEndpointNode{
							{
								ServerName: "D1",
								Endpoint:   "d1.endpoint",
								children: []*ServerEndpointNode{
									{
										ServerName: "E1",
										Endpoint:   "e1.endpoint",
										children:   nil,
									},
									{
										ServerName: "F1",
										Endpoint:   "f1.endpoint",
										children:   nil,
									},
								},
							},
						},
					},
				},
			},
		},
	}
	subTrees := subTreesWithEndpointRoot(&root)
	if subTrees == nil || len(subTrees) != 3 {
		t.Fatalf("subtrees length != 3:%+v", subTrees)
	}
}

func TestFindSubTreeEndpointPair(t *testing.T) {
	root := ServerEndpointNode{
		ServerName: "A",
		Endpoint:   "",
		children: []*ServerEndpointNode{
			{
				ServerName: "B2",
				Endpoint:   "",
				children: []*ServerEndpointNode{
					{
						ServerName: "C2",
						Endpoint:   "c2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "D2",
								Endpoint:   "d2.endpoint",
								children:   nil,
							},
						},
					},
					{
						ServerName: "E2",
						Endpoint:   "e2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "f2",
								Endpoint:   "f2.endpoint",
								children:   nil,
							},
						},
					},
				},
			},
			{
				ServerName: "B1",
				Endpoint:   "b1.endpoint",
				children: []*ServerEndpointNode{
					{
						ServerName: "C1",
						Endpoint:   "",
						children: []*ServerEndpointNode{
							{
								ServerName: "D1",
								Endpoint:   "d1.endpoint",
								children: []*ServerEndpointNode{
									{
										ServerName: "E1",
										Endpoint:   "e1.endpoint",
										children:   nil,
									},
									{
										ServerName: "F1",
										Endpoint:   "f1.endpoint",
										children:   nil,
									},
								},
							},
						},
					},
					{
						ServerName: "G1",
						Endpoint:   "G1.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "H1",
								Endpoint:   "h1.endpoint",
								children: []*ServerEndpointNode{
									{
										ServerName: "I1",
										Endpoint:   "i1.endpoint",
										children:   nil,
									},
								},
							},
						},
					},
				},
			},
		},
	}
	pairs := findTreeEndpointPair(&root)
	if pairs == nil || len(pairs) != 7 {
		t.Fatalf("subtrees length != 6:%+v", pairs)
	}
}
func TestServiceTopoDataWithEndpoint(t *testing.T) {
	root := ServerEndpointNode{
		ServerName: "A",
		Endpoint:   "",
		children: []*ServerEndpointNode{
			{
				ServerName: "B2",
				Endpoint:   "",
				children: []*ServerEndpointNode{
					{
						ServerName: "C2",
						Endpoint:   "c2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "D2",
								Endpoint:   "d2.endpoint",
								children:   nil,
							},
						},
					},
					{
						ServerName: "E2",
						Endpoint:   "e2.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "f2",
								Endpoint:   "f2.endpoint",
								children:   nil,
							},
						},
					},
				},
			},
			{
				ServerName: "B1",
				Endpoint:   "b1.endpoint",
				children: []*ServerEndpointNode{
					{
						ServerName: "C1",
						Endpoint:   "",
						children: []*ServerEndpointNode{
							{
								ServerName: "D1",
								Endpoint:   "d1.endpoint",
								children: []*ServerEndpointNode{
									{
										ServerName: "E1",
										Endpoint:   "e1.endpoint",
										children:   nil,
									},
									{
										ServerName: "F1",
										Endpoint:   "f1.endpoint",
										children:   nil,
									},
								},
							},
						},
					},
					{
						ServerName: "G1",
						Endpoint:   "G1.endpoint",
						children: []*ServerEndpointNode{
							{
								ServerName: "H1",
								Endpoint:   "h1.endpoint",
								children: []*ServerEndpointNode{
									{
										ServerName: "I1",
										Endpoint:   "i1.endpoint",
										children:   nil,
									},
								},
							},
						},
					},
				},
			},
		},
	}
	pairs := serviceTopoDataWithEndpoint("cmdbID", &root)
	if pairs == nil || len(pairs) != 7 {
		t.Fatalf("subtrees length != 6:%+v", pairs)
	}
}
